@@ -89,8 +89,8 @@ class GUI_EXPORT QgsColorButton : public QToolButton
 
     /**
      * Sets whether opacity modification (transparency) is permitted
-     * for the color. Defaults to false.
-     * \param allowOpacity set to true to allow opacity modification
+     * for the color. Defaults to FALSE.
+     * \param allowOpacity set to TRUE to allow opacity modification
      * \see allowOpacity()
      * \since QGIS 3.0
      */
@@ -99,7 +99,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
     /**
      * Returns whether opacity modification (transparency) is permitted
      * for the color.
-     * \returns true if opacity modification is allowed
+     * \returns TRUE if opacity modification is allowed
      * \see setAllowOpacity()
      * \since QGIS 3.0
      */
@@ -122,14 +122,14 @@ class GUI_EXPORT QgsColorButton : public QToolButton
     /**
      * Sets whether the drop-down menu should be shown for the button. The default behavior is to
      * show the menu.
-     * \param showMenu set to false to hide the drop-down menu
+     * \param showMenu set to FALSE to hide the drop-down menu
      * \see showMenu
      */
     void setShowMenu( bool showMenu );
 
     /**
      * Returns whether the drop-down menu is shown for the button.
-     * \returns true if drop-down menu is shown
+     * \returns TRUE if drop-down menu is shown
      * \see setShowMenu
      */
     bool showMenu() const { return menu() ? true : false; }
@@ -170,7 +170,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
     /**
      * Sets whether the "no color" option should be shown in the button's drop-down menu. If selected,
      * the "no color" option sets the color button's color to a totally transparent color.
-     * \param showNoColorOption set to true to show the no color option. This is disabled by default.
+     * \param showNoColorOption set to TRUE to show the no color option. This is disabled by default.
      * \see showNoColor
      * \see setNoColorString
      * \note The "no color" option is only shown if the color button is set to show an alpha channel in the color
@@ -181,7 +181,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
     /**
      * Returns whether the "no color" option is shown in the button's drop-down menu. If selected,
      * the "no color" option sets the color button's color to a totally transparent color.
-     * \returns true if the no color option is shown.
+     * \returns TRUE if the no color option is shown.
      * \see setShowNoColor
      * \see noColorString
      * \note The "no color" option is only shown if the color button is set to show an alpha channel in the color
@@ -201,12 +201,13 @@ class GUI_EXPORT QgsColorButton : public QToolButton
 
     /**
      * Sets whether a set to null (clear) option is shown in the button's drop-down menu.
-     * \param showNull set to true to show a null option
+     * \param showNull set to TRUE to show a null option
+     * \param nullString translated string to use for the null option. If not set, a default "Clear Color" string will be used.
      * \see showNull()
      * \see isNull()
      * \since QGIS 2.16
      */
-    void setShowNull( bool showNull );
+    void setShowNull( bool showNull, const QString &nullString = QString() );
 
     /**
      * Returns whether the set to null (clear) option is shown in the button's drop-down menu.
@@ -217,7 +218,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
     bool showNull() const;
 
     /**
-     * Returns true if the current color is null.
+     * Returns TRUE if the current color is null.
      * \see setShowNull()
      * \see showNull()
      * \since QGIS 2.16
@@ -297,7 +298,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
     /**
      * Creates an icon for displaying a \a color in a drop-down menu.
      *
-     * If \a showChecks set to true, then a checkboard pattern will be shown behind
+     * If \a showChecks set to TRUE, then a checkboard pattern will be shown behind
      * semi-transparent colors.
      *
      * \since QGIS 3.6
@@ -362,6 +363,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
      * Sets color to null.
      * \see setToDefaultColor()
      * \see setToNoColor()
+     * \see cleared()
      * \since QGIS 2.16
      */
     void setToNull();
@@ -379,7 +381,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
   signals:
 
     /**
-     * Is emitted whenever a new color is set for the button. The color is always valid.
+     * Emitted whenever a new color is set for the button. The color is always valid.
      * In case the new color is the same no signal is emitted, to avoid infinite loops.
      * \param color New color
      */
@@ -393,6 +395,14 @@ class GUI_EXPORT QgsColorButton : public QToolButton
      */
     void colorClicked( const QColor &color );
 
+    /**
+     * Emitted when the color is cleared (set to null).
+     *
+     * \see setToNull()
+     * \since QGIS 3.12
+     */
+    void cleared();
+
     /**
      * Emitted when the color is unlinked, e.g. when it was previously set to link
      * to a project color and is now no longer linked.
@@ -453,10 +463,6 @@ class GUI_EXPORT QgsColorButton : public QToolButton
 
   private:
 
-
-    QColor sampleColor( QPoint point ) const;
-
-    static QScreen *findScreenAt( QPoint pos );
     Behavior mBehavior = QgsColorButton::ShowDialog;
     QString mColorDialogTitle;
     QColor mColor;
@@ -476,6 +482,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
     bool mShowNoColorOption = false;
     QString mNoColorString;
     bool mShowNull = false;
+    QString mNullColorString;
 
     QPoint mDragStartPosition;
     bool mPickingColor = false;
@@ -489,7 +496,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
     /**
      * Attempts to parse mimeData as a color, either via the mime data's color data or by
      * parsing a textual representation of a color.
-     * \returns true if mime data could be intrepreted as a color
+     * \returns TRUE if mime data could be intrepreted as a color
      * \param mimeData mime data
      * \param resultColor QColor to store evaluated color
      * \see createColorMimeData
@@ -499,7 +506,7 @@ class GUI_EXPORT QgsColorButton : public QToolButton
     /**
      * Ends a color picking operation
      * \param eventPos global position of pixel to sample color from
-     * \param samplingColor set to true to actually sample the color, false to just cancel
+     * \param samplingColor set to TRUE to actually sample the color, FALSE to just cancel
      * the color picking operation
      */
     void stopPicking( QPoint eventPos, bool samplingColor = true );
