@@ -12,128 +12,21 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-#ifndef QGSGRADUATEDSYMBOLRENDERERV2_H
-#define QGSGRADUATEDSYMBOLRENDERERV2_H
+#ifndef QGSGRADUATEDSYMBOLRENDERER_H
+#define QGSGRADUATEDSYMBOLRENDERER_H
 
 #include "qgis_core.h"
 #include "qgis_sip.h"
 #include "qgis.h"
-#include "qgssymbol.h"
 #include "qgsrenderer.h"
-#include "qgsexpression.h"
-#include "qgsdatadefinedsizelegend.h"
-
-#include <QRegExp>
-
-
-/**
- * \ingroup core
- * \class QgsRendererRange
- */
-class CORE_EXPORT QgsRendererRange
-{
-  public:
-
-    /**
-     * Constructor for QgsRendererRange.
-     */
-    QgsRendererRange() = default;
-    QgsRendererRange( double lowerValue, double upperValue, QgsSymbol *symbol SIP_TRANSFER, const QString &label, bool render = true );
-    QgsRendererRange( const QgsRendererRange &range );
-
-    // default dtor is OK
-    QgsRendererRange &operator=( QgsRendererRange range );
-
-    bool operator<( const QgsRendererRange &other ) const;
-
-    double lowerValue() const;
-    double upperValue() const;
-
-    QgsSymbol *symbol() const;
-    QString label() const;
-
-    void setSymbol( QgsSymbol *s SIP_TRANSFER );
-    void setLabel( const QString &label );
-    void setLowerValue( double lowerValue );
-    void setUpperValue( double upperValue );
-
-    // \since QGIS 2.5
-    bool renderState() const;
-    void setRenderState( bool render );
-
-    // debugging
-    QString dump() const;
-
-    /**
-     * Creates a DOM element representing the range in SLD format.
-     * \param doc DOM document
-     * \param element destination DOM element
-     * \param props graduated renderer properties
-     * \param firstRange set to true if the range is the first range, where the lower value uses a <= test
-     * rather than a < test.
-     */
-    void toSld( QDomDocument &doc, QDomElement &element, QgsStringMap props, bool firstRange = false ) const;
-
-  protected:
-    double mLowerValue = 0, mUpperValue = 0;
-    std::unique_ptr<QgsSymbol> mSymbol;
-    QString mLabel;
-    bool mRender = true;
-
-    // for cpy+swap idiom
-    void swap( QgsRendererRange &other );
-};
-
-typedef QList<QgsRendererRange> QgsRangeList;
-
-
-/**
- * \ingroup core
- * \class QgsRendererRangeLabelFormat
- * \since QGIS 2.6
- */
-class CORE_EXPORT QgsRendererRangeLabelFormat
-{
-  public:
-    QgsRendererRangeLabelFormat();
-    QgsRendererRangeLabelFormat( const QString &format, int precision = 4, bool trimTrailingZeroes = false );
-
-    bool operator==( const QgsRendererRangeLabelFormat &other ) const;
-    bool operator!=( const QgsRendererRangeLabelFormat &other ) const;
-
-    QString format() const { return mFormat; }
-    void setFormat( const QString &format ) { mFormat = format; }
-
-    int precision() const { return mPrecision; }
-    void setPrecision( int precision );
-
-    bool trimTrailingZeroes() const { return mTrimTrailingZeroes; }
-    void setTrimTrailingZeroes( bool trimTrailingZeroes ) { mTrimTrailingZeroes = trimTrailingZeroes; }
-
-    //! \note labelForLowerUpper in Python bindings
-    QString labelForRange( double lower, double upper ) const SIP_PYNAME( labelForLowerUpper );
-    QString labelForRange( const QgsRendererRange &range ) const;
-    QString formatNumber( double value ) const;
-
-    void setFromDomElement( QDomElement &element );
-    void saveToDomElement( QDomElement &element );
-
-    static const int MAX_PRECISION;
-    static const int MIN_PRECISION;
-
-  protected:
-    QString mFormat;
-    int mPrecision = 4;
-    bool mTrimTrailingZeroes = false;
-    // values used to manage number formatting - precision and trailing zeroes
-    double mNumberScale = 1.0;
-    QString mNumberSuffix;
-    QRegExp mReTrailingZeroes;
-    QRegExp mReNegativeZero;
-};
+#include "qgsrendererrange.h"
+#include "qgsclassificationmethod.h"
 
 class QgsVectorLayer;
 class QgsColorRamp;
+class QgsDataDefinedSizeLegend;
+class QgsSymbol;
+class QgsExpression;
 
 /**
  * \ingroup core
@@ -147,16 +40,18 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
 
     ~QgsGraduatedSymbolRenderer() override;
 
-    QgsSymbol *symbolForFeature( QgsFeature &feature, QgsRenderContext &context ) override;
-    QgsSymbol *originalSymbolForFeature( QgsFeature &feature, QgsRenderContext &context ) override;
+    QgsSymbol *symbolForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
+    QgsSymbol *originalSymbolForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
     void startRender( QgsRenderContext &context, const QgsFields &fields ) override;
     void stopRender( QgsRenderContext &context ) override;
     QSet<QString> usedAttributes( const QgsRenderContext &context ) const override;
+    bool filterNeedsGeometry() const override;
     QString dump() const override;
     QgsGraduatedSymbolRenderer *clone() const override SIP_FACTORY;
     void toSld( QDomDocument &doc, QDomElement &element, const QgsStringMap &props = QgsStringMap() ) const override;
     QgsFeatureRenderer::Capabilities capabilities() override { return SymbolLevels | Filter; }
-    QgsSymbolList symbols( QgsRenderContext &context ) override;
+    QgsSymbolList symbols( QgsRenderContext &context ) const override;
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const override;
 
     QString classAttribute() const { return mAttrName; }
     void setClassAttribute( const QString &attr ) { mAttrName = attr; }
@@ -180,7 +75,7 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
      * Add a breakpoint by splitting existing classes so that the specified
      * value becomes a break between two classes.
      * \param breakValue position to insert break
-     * \param updateSymbols set to true to reapply ramp colors to the new
+     * \param updateSymbols set to TRUE to reapply ramp colors to the new
      * symbol ranges
      * \since QGIS 2.9
      */
@@ -194,14 +89,14 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
 
     /**
      * Tests whether classes assigned to the renderer have ranges which overlap.
-     * \returns true if ranges overlap
+     * \returns TRUE if ranges overlap
      * \since QGIS 2.10
      */
     bool rangesOverlap() const;
 
     /**
      * Tests whether classes assigned to the renderer have gaps between the ranges.
-     * \returns true if ranges have gaps
+     * \returns TRUE if ranges have gaps
      * \since QGIS 2.10
      */
     bool rangesHaveGaps() const;
@@ -209,6 +104,23 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
     void sortByValue( Qt::SortOrder order = Qt::AscendingOrder );
     void sortByLabel( Qt::SortOrder order = Qt::AscendingOrder );
 
+    /**
+     * Returns the classification method
+     * \since QGIS 3.10
+     */
+    QgsClassificationMethod *classificationMethod() const;
+
+    /**
+     * Defines the classification method
+     * This will take ownership of the method
+     * \since QGIS 3.10
+     */
+    void setClassificationMethod( QgsClassificationMethod *method SIP_TRANSFER );
+
+    /**
+      * Classification mode
+      * \deprecated since QGIS 3.10 use QgsClassificationMethod::MethodId instead
+      */
     enum Mode
     {
       EqualInterval,
@@ -218,40 +130,131 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
       Pretty,
       Custom
     };
+    // TODO QGIS 4: remove
+    // this could not be tagged with Q_DECL_DEPRECATED due to Doxygen warning
+    // might be fixed in newer Doxygen (does not on 1.8.13, might be ok on 1.8.16)
 
-    Mode mode() const { return mMode; }
-    void setMode( Mode mode ) { mMode = mode; }
+
+    //! \deprecated since QGIS 3.10 use classficationMethod instead
+    Q_DECL_DEPRECATED Mode mode() const SIP_DEPRECATED { return modeFromMethodId( mClassificationMethod->id() ); }
+    //! \deprecated since QGIS 3.10 use classficationMethod instead
+    Q_DECL_DEPRECATED void setMode( Mode mode ) SIP_DEPRECATED;
+
+    /**
+     * Returns if we want to classify symmetric around a given value
+     * \since QGIS 3.4
+     * \deprecated since QGIS 3.10 use classficationMethod instead
+     */
+    Q_DECL_DEPRECATED bool useSymmetricMode() const SIP_DEPRECATED { return mClassificationMethod->symmetricModeEnabled(); }
+
+    /**
+     * Set if we want to classify symmetric around a given value
+     * \since QGIS 3.4
+     * \deprecated since QGIS 3.10 use classficationMethod instead
+     */
+    Q_DECL_DEPRECATED  void setUseSymmetricMode( bool useSymmetricMode ) SIP_DEPRECATED;
+
+    /**
+     * Returns the pivot value for symmetric classification
+     * \since QGIS 3.4
+     * \deprecated since QGIS 3.10 use classficationMethod instead
+     */
+    Q_DECL_DEPRECATED double symmetryPoint() const SIP_DEPRECATED { return mClassificationMethod->symmetryPoint(); }
+
+    /**
+     * Set the pivot point
+     * \since QGIS 3.4
+     * \deprecated since QGIS 3.10 use classficationMethod instead
+     */
+    Q_DECL_DEPRECATED void setSymmetryPoint( double symmetryPoint ) SIP_DEPRECATED;
+
+
+    /**
+     * Returns if we want to have a central class astride the pivot value
+     * \since QGIS 3.4
+     * \deprecated since QGIS 3.10 use classficationMethod instead
+     */
+    Q_DECL_DEPRECATED bool astride() const SIP_DEPRECATED { return mClassificationMethod->symmetryAstride(); }
+
+    /**
+     * Set if we want a central class astride the pivot value
+     * \since QGIS 3.4
+     * \deprecated since QGIS 3.10 use classficationMethod instead
+     */
+    Q_DECL_DEPRECATED void setAstride( bool astride ) SIP_DEPRECATED;
+
+    /**
+     * Remove the breaks that are above the existing opposite sign classes to keep colors symmetrically balanced around symmetryPoint
+     * Does not put a break on the symmetryPoint. This is done before.
+     * \param breaks The breaks of an already-done classification
+     * \param symmetryPoint The point around which we want a symmetry
+     * \param astride A bool indicating if the symmetry is made astride the symmetryPoint or not ( [-1,1] vs. [-1,0][0,1] )
+     * \since QGIS 3.4
+     * \deprecated since QGIS 3.10, use QgsClassificationMethod::makeBreaksSymmetric instead
+     */
+    Q_DECL_DEPRECATED static void makeBreaksSymmetric( QList<double> &breaks SIP_INOUT, double symmetryPoint, bool astride ) SIP_DEPRECATED;
+
+    /**
+     * Compute the equal interval classification
+     * \param minimum The minimum value of the distribution
+     * \param maximum The maximum value of the distribution
+     * \param classes The number of classes desired
+     * \param useSymmetricMode A bool indicating if we want to have classes and hence colors ramp symmetric around a value
+     * \param symmetryPoint The point around which we want a symmetry
+     * \param astride A bool indicating if the symmetry is made astride the symmetryPoint or not ( [-1,1] vs. [-1,0][0,1] )
+     * \deprecated since QGIS 3.10 use QgsClassificationEqualInterval class instead
+     */
+    Q_DECL_DEPRECATED static QList<double> calcEqualIntervalBreaks( double minimum, double maximum, int classes, bool useSymmetricMode, double symmetryPoint, bool astride ) SIP_DEPRECATED;
 
     /**
      * Recalculate classes for a layer
      * \param vlayer  The layer being rendered (from which data values are calculated)
      * \param mode    The calculation mode
      * \param nclasses The number of classes to calculate (approximate for some modes)
-     * \since QGIS 2.6
+     * \param useSymmetricMode A bool indicating if we want to have classes and hence colors ramp symmetric around a value
+     * \param symmetryPoint The value around which the classes will be symmetric if useSymmetricMode is checked
+     * \param astride A bool indicating if the symmetry is made astride the symmetryPoint or not ( [-1,1] vs. [-1,0][0,1] )
+     * \since QGIS 2.6 (three first arguments) and 3.4 (three last arguments)
+     * \deprecated since QGIS 3.10
      */
-    void updateClasses( QgsVectorLayer *vlayer, Mode mode, int nclasses );
+    Q_DECL_DEPRECATED void updateClasses( QgsVectorLayer *vlayer, Mode mode, int nclasses, bool useSymmetricMode = false, double symmetryPoint = 0.0, bool astride = false ) SIP_DEPRECATED;
 
     /**
-     * Return the label format used to generate default classification labels
-     * \since QGIS 2.6
+     * Recalculate classes for a layer
+     * \param vl  The layer being rendered (from which data values are calculated)
+     * \param nclasses the number of classes
      */
-    const QgsRendererRangeLabelFormat &labelFormat() const { return mLabelFormat; }
+    void updateClasses( const QgsVectorLayer *vl, int nclasses );
+
+    Q_NOWARN_DEPRECATED_PUSH;
+
+    /**
+     * Returns the label format used to generate default classification labels
+     * \since QGIS 2.6
+     * \deprecated since QGIS 3.10 use classificationMethod() and QgsClassificationMethod::setLabelFormat instead
+     */
+    Q_DECL_DEPRECATED QgsRendererRangeLabelFormat labelFormat() const SIP_DEPRECATED;
 
     /**
      * Set the label format used to generate default classification labels
      * \param labelFormat The string appended to classification labels
-     * \param updateRanges If true then ranges ending with the old unit string are updated to the new.
+     * \param updateRanges If TRUE then ranges ending with the old unit string are updated to the new.
      * \since QGIS 2.6
+     * \deprecated since QGIS 3.10 use classificationMethod() and QgsClassificationMethod::setLabelFormat instead
      */
-    void setLabelFormat( const QgsRendererRangeLabelFormat &labelFormat, bool updateRanges = false );
+    Q_DECL_DEPRECATED void setLabelFormat( const QgsRendererRangeLabelFormat &labelFormat, bool updateRanges = false ) SIP_DEPRECATED;
+
+    Q_NOWARN_DEPRECATED_POP;
 
     /**
      * Reset the label decimal places to a numberbased on the minimum class interval
-     * \param updateRanges if true then ranges currently using the default label will be updated
+     * \param updateRanges if TRUE then ranges currently using the default label will be updated
      * \since QGIS 2.6
      */
     void calculateLabelPrecision( bool updateRanges = true );
 
+    Q_NOWARN_DEPRECATED_PUSH;
+
     /**
      * Creates a new graduated renderer.
      * \param vlayer vector layer
@@ -261,22 +264,32 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
      * \param symbol base symbol
      * \param ramp color ramp for classes
      * \param legendFormat
+     * \param useSymmetricMode A bool indicating if we want to have classes and hence colors ramp symmetric around a value
+     * \param symmetryPoint The value around which the classes will be symmetric if useSymmetricMode is checked
+     * \param listForCboPrettyBreaks The list of potential pivot values for symmetric mode with prettybreaks mode
+     * \param astride A bool indicating if the symmetry is made astride the symmetryPoint or not ( [-1,1] vs. [-1,0][0,1] )
      * \returns new QgsGraduatedSymbolRenderer object
+     * \deprecated since QGIS 3.10
      */
-    static QgsGraduatedSymbolRenderer *createRenderer( QgsVectorLayer *vlayer,
+    static Q_DECL_DEPRECATED QgsGraduatedSymbolRenderer *createRenderer( QgsVectorLayer *vlayer,
         const QString &attrName,
         int classes,
         Mode mode,
         QgsSymbol *symbol SIP_TRANSFER,
         QgsColorRamp *ramp SIP_TRANSFER,
-        const QgsRendererRangeLabelFormat &legendFormat = QgsRendererRangeLabelFormat() );
+        const QgsRendererRangeLabelFormat &legendFormat = QgsRendererRangeLabelFormat(),
+        bool useSymmetricMode = false,
+        double symmetryPoint = 0.0,
+        const QStringList &listForCboPrettyBreaks = QStringList(),
+        bool astride = false ) SIP_DEPRECATED;
+    Q_NOWARN_DEPRECATED_POP;
 
     //! create renderer from XML element
     static QgsFeatureRenderer *create( QDomElement &element, const QgsReadWriteContext &context ) SIP_FACTORY;
 
     QDomElement save( QDomDocument &doc, const QgsReadWriteContext &context ) override;
     QgsLegendSymbolList legendSymbolItems() const override;
-    QSet< QString > legendKeysForFeature( QgsFeature &feature, QgsRenderContext &context ) override;
+    QSet< QString > legendKeysForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
 
     /**
      * Returns the renderer's source symbol, which is the base symbol used for the each classes' symbol before applying
@@ -286,6 +299,15 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
      */
     QgsSymbol *sourceSymbol();
 
+    /**
+     * Returns the renderer's source symbol, which is the base symbol used for the each classes' symbol before applying
+     * the classes' color.
+     * \see setSourceSymbol()
+     * \see sourceColorRamp()
+     * \note Not available in Python bindings.
+     */
+    const QgsSymbol *sourceSymbol() const SIP_SKIP;
+
     /**
      * Sets the source symbol for the renderer, which is the base symbol used for the each classes' symbol before applying
      * the classes' color.
@@ -302,6 +324,14 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
      */
     QgsColorRamp *sourceColorRamp();
 
+    /**
+     * Returns the source color ramp, from which each classes' color is derived.
+     * \see setSourceColorRamp()
+     * \see sourceSymbol()
+     * \note Not available in Python bindings.
+     */
+    const QgsColorRamp *sourceColorRamp() const SIP_SKIP;
+
     /**
      * Sets the source color ramp.
      * \param ramp color ramp. Ownership is transferred to the renderer
@@ -331,13 +361,13 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
     void setSymbolSizes( double minSize, double maxSize );
 
     /**
-     * return the min symbol size when graduated by size
+     * Returns the min symbol size when graduated by size
      * \since QGIS 2.10
      */
     double minSymbolSize() const;
 
     /**
-     * return the max symbol size when graduated by size
+     * Returns the max symbol size when graduated by size
      * \since QGIS 2.10
      */
     double maxSymbolSize() const;
@@ -349,7 +379,7 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
     };
 
     /**
-     * return the method used for graduation (either size or color)
+     * Returns the method used for graduation (either size or color)
      * \since QGIS 2.10
      */
     GraduatedMethod graduatedMethod() const { return mGraduatedMethod; }
@@ -379,7 +409,7 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
      * different symbol sizes collapsed in one legend node or separated across multiple legend nodes etc.
      *
      * When renderer does not use data-defined size or does not use marker symbols, these settings will be ignored.
-     * Takes ownership of the passed settings objects. Null pointer is a valid input that disables data-defined
+     * Takes ownership of the passed settings objects. NULLPTR is a valid input that disables data-defined
      * size legend.
      * \since QGIS 3.0
      */
@@ -387,28 +417,44 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
 
     /**
      * Returns configuration of appearance of legend when using data-defined size for marker symbols.
-     * Will return null if the functionality is disabled.
+     * Will return NULLPTR if the functionality is disabled.
      * \since QGIS 3.0
      */
     QgsDataDefinedSizeLegend *dataDefinedSizeLegend() const;
 
+    /**
+     * Updates the labels of the ranges
+     * \since QGIS 3.10
+     */
+    void updateRangeLabels();
+
+    /**
+     * Returns the renderer range matching the provided \a value, or NULLPTR if no
+     * range matches the value.
+     *
+     * \since QGIS 3.10.1
+     */
+    const QgsRendererRange *rangeForValue( double value ) const;
+
   protected:
     QString mAttrName;
     QgsRangeList mRanges;
-    Mode mMode = Custom;
     std::unique_ptr<QgsSymbol> mSourceSymbol;
     std::unique_ptr<QgsColorRamp> mSourceColorRamp;
-    QgsRendererRangeLabelFormat mLabelFormat;
 
     std::unique_ptr<QgsExpression> mExpression;
     GraduatedMethod mGraduatedMethod = GraduatedColor;
     //! attribute index (derived from attribute name in startRender)
+
     int mAttrNum = -1;
     bool mCounting = false;
 
     std::unique_ptr<QgsDataDefinedSizeLegend> mDataDefinedSizeLegend;
 
-    QgsSymbol *symbolForValue( double value );
+    /**
+     * Gets the symbol which is used to represent \a value.
+     */
+    QgsSymbol *symbolForValue( double value ) const;
 
     /**
      * Returns the matching legend key for a value.
@@ -416,23 +462,33 @@ class CORE_EXPORT QgsGraduatedSymbolRenderer : public QgsFeatureRenderer
     QString legendKeyForValue( double value ) const;
 
     //! \note not available in Python bindings
-    static const char *graduatedMethodStr( GraduatedMethod method ) SIP_SKIP;
+    static QString graduatedMethodStr( GraduatedMethod method ) SIP_SKIP;
+
+    std::shared_ptr<QgsClassificationMethod> mClassificationMethod;
 
   private:
 
     /**
      * Returns calculated value used for classifying a feature.
      */
-    QVariant valueForFeature( QgsFeature &feature, QgsRenderContext &context ) const;
+    QVariant valueForFeature( const QgsFeature &feature, QgsRenderContext &context ) const;
 
     //! Returns list of legend symbol items from individual ranges
     QgsLegendSymbolList baseLegendSymbolItems() const;
 
+    // TODO QGIS 4: remove
+    Q_NOWARN_DEPRECATED_PUSH
+    static QString methodIdFromMode( QgsGraduatedSymbolRenderer::Mode mode );
+    static QgsGraduatedSymbolRenderer::Mode modeFromMethodId( const QString &methodId );
+    Q_NOWARN_DEPRECATED_POP
+
 #ifdef SIP_RUN
     QgsGraduatedSymbolRenderer( const QgsGraduatedSymbolRenderer & );
     QgsGraduatedSymbolRenderer &operator=( const QgsGraduatedSymbolRenderer & );
 #endif
 
+    friend class TestQgsGraduatedSymbolRenderer;
+
 };
 
-#endif // QGSGRADUATEDSYMBOLRENDERERV2_H
+#endif // QGSGRADUATEDSYMBOLRENDERER_H
