@@ -33,6 +33,7 @@
 #include "qgsmaplayer.h"
 #include "qgsgeometry.h"
 #include "qgstemporalrangeobject.h"
+#include "qgsmapclippingregion.h"
 
 class QPainter;
 
@@ -311,6 +312,8 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
       RenderPartialOutput      = 0x200, //!< Whether to make extra effort to update map image with partially rendered layers (better for interactive map canvas). Added in QGIS 3.0
       RenderPreviewJob         = 0x400, //!< Render is a 'canvas preview' render, and shortcuts should be taken to ensure fast rendering
       RenderBlocking           = 0x800, //!< Render and load remote sources in the same thread to ensure rendering remote sources (svg and images). WARNING: this flag must NEVER be used from GUI based applications (like the main QGIS application) or crashes will result. Only for use in external scripts or QGIS server.
+      LosslessImageRendering   = 0x1000, //!< Render images losslessly whenever possible, instead of the default lossy jpeg rendering used for some destination devices (e.g. PDF). This flag only works with builds based on Qt 5.13 or later.
+      Render3DMap              = 0x2000, //!< Render is for a 3D map
       // TODO: ignore scale-based visibility (overview)
     };
     Q_DECLARE_FLAGS( Flags, Flag )
@@ -460,6 +463,13 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      */
     QgsPointXY layerToMapCoordinates( const QgsMapLayer *layer, QgsPointXY point ) const;
 
+    /**
+     * \brief transform point coordinates from layer's CRS to output CRS
+     * \returns the transformed point
+     * \since QGIS 3.16
+     */
+    QgsPoint layerToMapCoordinates( const QgsMapLayer *layer, const QgsPoint &point ) const;
+
     /**
      * \brief transform rectangle from layer's CRS to output CRS
      * \see layerExtentToOutputExtent() if you want to transform a bounding box
@@ -473,6 +483,13 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      */
     QgsPointXY mapToLayerCoordinates( const QgsMapLayer *layer, QgsPointXY point ) const;
 
+    /**
+     * \brief transform point coordinates from output CRS to layer's CRS
+     * \returns the transformed point
+     * \since QGIS 3.16
+     */
+    QgsPoint mapToLayerCoordinates( const QgsMapLayer *layer, const QgsPoint &point ) const;
+
     /**
      * \brief transform rectangle from output CRS to layer's CRS
      * \see outputExtentToLayerExtent() if you want to transform a bounding box
@@ -497,14 +514,16 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
 
     /**
      * Sets the segmentation tolerance applied when rendering curved geometries
-    \param tolerance the segmentation tolerance*/
+     * \param tolerance the segmentation tolerance
+    */
     void setSegmentationTolerance( double tolerance ) { mSegmentationTolerance = tolerance; }
     //! Gets the segmentation tolerance applied when rendering curved geometries
     double segmentationTolerance() const { return mSegmentationTolerance; }
 
     /**
      * Sets segmentation tolerance type (maximum angle or maximum difference between curve and approximation)
-    \param type the segmentation tolerance typename*/
+     * \param type the segmentation tolerance typename
+    */
     void setSegmentationToleranceType( QgsAbstractGeometry::SegmentationToleranceType type ) { mSegmentationToleranceType = type; }
     //! Gets segmentation tolerance type (maximum angle or maximum difference between curve and approximation)
     QgsAbstractGeometry::SegmentationToleranceType segmentationToleranceType() const { return mSegmentationToleranceType; }
@@ -578,6 +597,36 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
      */
     QList< QgsLabelBlockingRegion > labelBlockingRegions() const { return mLabelBlockingRegions; }
 
+    /**
+     * Adds a new clipping \a region to the map settings.
+     *
+     * \see clippingRegions()
+     * \see setClippingRegions()
+     *
+     * \since QGIS 3.16
+     */
+    void addClippingRegion( const QgsMapClippingRegion &region );
+
+    /**
+     * Sets the list of clipping \a regions to apply to the map.
+     *
+     * \see addClippingRegion()
+     * \see clippingRegions()
+     *
+     * \since QGIS 3.16
+     */
+    void setClippingRegions( const QList< QgsMapClippingRegion > &regions );
+
+    /**
+     * Returns the list of clipping regions to apply to the map.
+     *
+     * \see addClippingRegion()
+     * \see setClippingRegions()
+     *
+     * \since QGIS 3.16
+     */
+    QList< QgsMapClippingRegion > clippingRegions() const;
+
     /**
      * Sets the simplification setting to use when rendering vector layers.
      *
@@ -692,6 +741,7 @@ class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
   private:
 
     QList< QgsLabelBlockingRegion > mLabelBlockingRegions;
+    QList< QgsMapClippingRegion > mClippingRegions;
     QList< QgsRenderedFeatureHandlerInterface * > mRenderedFeatureHandlers;
 };
 
